@@ -40,7 +40,13 @@ from pg8000.converters import (
     VARCHAR_ARRAY,
     XID,
 )
-from pg8000.core import Context, CoreConnection, ver
+from pg8000.core import (
+    Context,
+    CoreConnection,
+    IN_FAILED_TRANSACTION,
+    IN_TRANSACTION,
+    ver,
+)
 from pg8000.exceptions import DatabaseError, Error, InterfaceError
 
 
@@ -441,7 +447,7 @@ class Cursor:
             .. versionadded:: 1.9.11
         """
         try:
-            if not self._c.in_transaction and not self._c.autocommit:
+            if not self._c._in_transaction and not self._c.autocommit:
                 self._c.execute_simple("begin transaction")
 
             if len(args) == 0 and stream is None:
@@ -489,7 +495,7 @@ class Cursor:
             rowcounts.append(self._context.row_count)
 
         if len(rowcounts) == 0:
-            self._context = Context()
+            self._context = Context(None)
         elif -1 in rowcounts:
             self._context.row_count = -1
         else:
@@ -644,6 +650,10 @@ class Connection(CoreConnection):
         warn(f"DB-API extension connection.{error.__name__} used", stacklevel=3)
         return error
 
+    @property
+    def _in_transaction(self):
+        return self._transaction_status in (IN_TRANSACTION, IN_FAILED_TRANSACTION)
+
     def cursor(self):
         """Creates a :class:`Cursor` object bound to this
         connection.
@@ -667,7 +677,7 @@ class Connection(CoreConnection):
         This function is part of the `DBAPI 2.0 specification
         <http://www.python.org/dev/peps/pep-0249/>`_.
         """
-        if not self.in_transaction:
+        if not self._in_transaction:
             return
         self.execute_unnamed("rollback")
 
